@@ -184,7 +184,7 @@
                   proximity = as.double(proxmatrix),
                   nodes = as.integer(nodes),
                   nodexts = as.integer(nodexts),
-                  DUP=FALSE,
+                  #DUP=FALSE,
                   PACKAGE = "randomForest")[keepIndex]
             ## Apply bias correction if needed.
             yhat <- rep(NA, length(rn))
@@ -239,7 +239,7 @@
                  prox = as.integer(proximity),
                  proxmatrix = as.double(proxmatrix),
                  nodes = as.integer(nodes),
-                 DUP=FALSE,
+                 #DUP=FALSE,
                  PACKAGE = "randomForest")
         if (out.type > 1) {
             out.class.votes <- t(matrix(t1$countts, nrow = nclass, ncol = ntest))
