@@ -3,7 +3,7 @@
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
-   as published by the Free Software Foundation; either version 2
+   as published by the R_Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -305,7 +305,7 @@ void F77_NAME(catmax)(double *parentDen, double *tclasscat,
     int j, k, n, icat[MAX_CAT], nsplit;
     double leftNum, leftDen, rightNum, decGini, *leftCatClassCount;
 
-    leftCatClassCount = (double *) Calloc(*nclass, double);
+    leftCatClassCount = (double *) R_Calloc(*nclass, double);
     *nhit = 0;
     nsplit = *lcat > *ncmax ?
         *ncsplit : (int) pow(2.0, (double) *lcat - 1) - 1;
@@ -348,7 +348,7 @@ void F77_NAME(catmax)(double *parentDen, double *tclasscat,
             *catsp = *lcat > *ncmax ? pack(*lcat, icat) : n + 1;
         }
     }
-    Free(leftCatClassCount);
+    R_Free(leftCatClassCount);
 }
 
 
@@ -424,7 +424,7 @@ void predictClassTree(double *x, int n, int mdim, int *treemap,
 
     /* decode the categorical splits */
     if (maxcat > 1) {
-        cbestsplit = (int *) Calloc(maxcat * treeSize, int);
+        cbestsplit = (int *) R_Calloc(maxcat * treeSize, int);
         zeroInt(cbestsplit, maxcat * treeSize);
         for (i = 0; i < treeSize; ++i) {
             if (nodestatus[i] != NODE_TERMINAL) {
@@ -458,5 +458,5 @@ void predictClassTree(double *x, int n, int mdim, int *treemap,
 		jts[i] = nodeclass[k];
 		nodex[i] = k + 1;
     }
-    if (maxcat > 1) Free(cbestsplit);
+    if (maxcat > 1) R_Free(cbestsplit);
 }
