@@ -3,7 +3,7 @@
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
-   as published by the Free Software Foundation; either version 2
+   as published by the R_Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -64,8 +64,8 @@ void makeA(double *x, const int mdim, const int nsample, int *cat, int *a,
     int i, j, n1, n2, *index;
     double *v;
 
-    v     = (double *) Calloc(nsample, double);
-    index = (int *) Calloc(nsample, int);
+    v     = (double *) R_Calloc(nsample, double);
+    index = (int *) R_Calloc(nsample, int);
 
     for (i = 0; i < mdim; ++i) {
         if (cat[i] == 1) { /* numerical predictor */
@@ -92,8 +92,8 @@ void makeA(double *x, const int mdim, const int nsample, int *cat, int *a,
                 a[i + j*mdim] = (int) x[i + j * mdim];
         }
     }
-    Free(index);
-    Free(v);
+    R_Free(index);
+    R_Free(v);
 }
 
 
@@ -185,7 +185,7 @@ void permuteOOB(int m, double *x, int *in, int nsample, int mdim) {
     double *tp, tmp;
     int i, last, k, nOOB = 0;
 
-    tp = (double *) Calloc(nsample, double);
+    tp = (double *) R_Calloc(nsample, double);
 
     for (i = 0; i < nsample; ++i) {
 		/* make a copy of the OOB part of the data into tp (for permuting) */
@@ -212,7 +212,7 @@ void permuteOOB(int m, double *x, int *in, int nsample, int mdim) {
             nOOB++;
 		}
     }
-    Free(tp);
+    R_Free(tp);
 }
 
 /* Compute proximity. */
@@ -282,3 +282,130 @@ void F77_NAME(unpack)(double *pack, int *nBits, int *bits) {
 	unpack(*pack, *nBits, bits);
 }
 
+void sampleDataRows(int populationSize, int sampleSize, int useWeights, 
+                    int withReplacement, double *weights, int *sampledIndices) {
+    /*At this point shouldn't need to check for sanity (sample > population)*/
+    /*Rprintf("Flags: replace %d useweights %d ", withReplacement, useWeights);*/
+    if(withReplacement) {
+        
+        if(useWeights) {
+            sampleWithReplacementWithWeights(sampleSize, populationSize, weights, sampledIndices);
+        }
+        else{
+            /*Rprintf("We will be sampling with replacement and no weights");*/
+            sampleWithReplacement(sampleSize, populationSize, sampledIndices);
+        }
+        
+    }
+    else{
+        if(useWeights) {
+            sampleWithoutReplacementWithWeights(sampleSize, populationSize, weights, sampledIndices);
+        }
+        else{
+            sampleWithoutReplacement(sampleSize, populationSize, sampledIndices);
+        }
+    }
+}
+
+void sampleWithReplacementWithWeights(int sampleSize, int populationSize, double *weights, int *sampledIndices) {
+    int mflag = 0;
+    int numBoundaries = populationSize + 1;
+    double *boundaries = (double*)R_Calloc(numBoundaries, double);
+    calculateBoundaries(weights, boundaries, populationSize, numBoundaries);
+
+    double sample;
+    for(int i = 0; i < sampleSize; ++i) {
+        sample = unif_rand();
+        sampledIndices[i] = findInterval(boundaries, numBoundaries, sample, 1, 1,0, &mflag) - 1;
+    }
+}
+
+void sampleWithoutReplacementWithWeights(int sampleSize, int populationSize, double *weights, int *sampledIndices) {
+    /*A snippet to run once when rf is run to verify the output of find_interval, which is what we use to sample with weights*/
+    /*if(tree == 0){
+        int testflag = 0;
+        Rprintf("Testing the find interval function in r\n");
+        double testBoundaries[] = {0, .2, .6, .6, .6, .6, .8, .8, .8, 1};
+        int testInterval = findInterval(testBoundaries, 11, .7, 1, 1,0, &testflag);
+        Rprintf("The resultant test interval found was %d\n", testInterval);
+    }*/
+    int mflag = 0;
+    int numBoundaries = populationSize + 1;
+    double *weightsForTree = (double*)R_Calloc(populationSize, double);
+    
+    /*copy over weights to array so we don't mess with original weights when removing them*/
+    for(int i = 0; i < populationSize; i++){
+        weightsForTree[i] = weights[i];
+    }
+    double *boundaries = (double*)R_Calloc(numBoundaries, double);
+    int *populationTaken = (int*)R_Calloc(populationSize, int);
+    zeroInt(populationTaken, populationSize);
+    calculateBoundaries(weightsForTree, boundaries, populationSize, numBoundaries);
+    int first;
+    int sampledIndex;
+    int sampledIndexCopy;
+    double sample;
+    
+    for(int i = 0; i < sampleSize; ++i) {
+        sample = unif_rand();
+        sampledIndex = findInterval(boundaries, numBoundaries, sample, 1, 1,0, &mflag) - 1;
+        first = sampledIndex;
+        sampledIndexCopy = sampledIndex;
+        sampledIndices[i] = sampledIndex;
+        populationTaken[sampledIndex] = -1;
+        removeWeightAndNormalize(weightsForTree, sampledIndex, populationSize);
+        calculateBoundaries(weightsForTree, boundaries, populationSize, numBoundaries);
+    }
+}
+
+void sampleWithReplacement(int sampleSize, int populationSize, int *sampledIndices) {
+    for(int i = 0; i < sampleSize; ++i) {
+        sampledIndices[i] = unif_rand() * populationSize;
+    }
+}
+
+void sampleWithoutReplacement(int sampleSize, int populationSize, int *sampledIndices) {
+    int *indices = (int*)R_Calloc(populationSize, int);
+    for(int i = 0; i < populationSize; ++i) {
+        indices[i] = i;
+    }
+    int indexOfLast = populationSize - 1;
+    int k;
+    for(int i = 0; i < sampleSize; i++) {
+        int currentSample = unif_rand() * (indexOfLast + 1);
+        k = indices[currentSample];
+        swapInt(indices[currentSample], indices[indexOfLast]);
+        indexOfLast--;
+        sampledIndices[i] = k;
+    }
+}
+
+void removeWeightAndNormalize(double *weights, int indexToRemove, int populationSize) {
+    double weightToRemove = weights[indexToRemove];
+    double sumOfWeights = 1 - weightToRemove;
+    weights[indexToRemove] = 0;
+    
+    for(int i = 0; i < populationSize; i++) {
+        weights[i] /= sumOfWeights;
+    }
+}
+
+void normalizeWeights(double *weights, int numWeights){
+    double weightSum = 0;
+    for(int i = 0; i < numWeights; i++){
+        weightSum += weights[i];
+    }
+    for(int i = 0; i < numWeights; i++){
+        weights[i] /= weightSum;
+    }
+}
+
+void calculateBoundaries(double *weights, double *boundaries, int populationSize, int numBoundaries) {
+    zeroDouble(boundaries, numBoundaries);
+    double currentSum = 0;
+    boundaries[0] = currentSum;
+    for(int i = 1; i <= populationSize; i++) {
+        currentSum += weights[i - 1];
+        boundaries[i] = currentSum;
+    }
+}
